#
# Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019. Built in San Francisco.
#
# This software is distributed under commercial non-GPL license for personal, educational,
# corporate or any other use. The software as a whole or any parts of that are prohibited
# for distribution and/or use without obtaining license from 3D Control Systems, Inc.
#
# If you do not have the license to use this software, please delete all software files
# immediately and contact sales to obtain the license: sales@3dprinteros.com.
# If you are unsure about the licensing please contact directly our sales: sales@3dprinteros.com.

import sys
import time
import logging

try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO

import log
import config
import http_client
from dual_cam import Camera


class M5thCamera(Camera):
    def __init__(self, parent):
        self.logger = logging.getLogger(__name__)
        self.image_extension = config.get_settings()["camera"]["img_ext"]
        self.image_quality = config.get_settings()["camera"]["img_qual"]
        self.hardware_resize = config.get_settings()["camera"]["hardware_resize"]
        self.min_loop_time = 1  # config.get_settings()["camera"]["min_loop_time"]
        self.stop_flag = False
        self.parent = parent
        self.http_client = http_client.HTTPClient(self, keep_connection_flag=True)
        self.user_token = parent.parent.parent.user_login.user_token
        self.http_client.mac = parent.parent.parent.user_login.mac
        self.ip = parent.ip
        import numpy as np
        import cv2 as cv2
        self.np = np
        self.cv2 = cv2

    @log.log_exception
    def main_loop(self):
        if not self.ip:
            self.logger.error('No IP. Stopping...')
            return
        self.makerbot = self.parent.makerbot
        number = self.get_number_by_ip()
        time.sleep(self.min_loop_time)
        while not self.stop_flag and not self.parent.stop_flag:
            self.logger.info("camera %s cycle" % number)
            frame_start_time = time.time()
            frame = self.make_shot()
            if frame:
                # self.logger.debug("Got frame from camera N" + str(number))
                self.send_frame(number, frame)
            else:
                self.logger.warning("No frame from camera N" + str(number))
            while time.time() < frame_start_time + 0.001 + self.min_loop_time:
                time.sleep(0.01)
        self.http_client.close()
        sys.exit(0)

    def get_number_by_ip(self):
        data = self.ip.split(".")
        for i, item in enumerate(data):
            data[i] = item.zfill(3)
        return int(''.join(data))-1

    def make_shot(self):
        _, width, height, type, image_data = self.makerbot._get_raw_camera_image_data()
        if not image_data:
            self.logger.warning('Failed to get camera frame: %i, %i' % (width, height))
            return None
        if type == 1:
            encode_param = [int(self.cv2.IMWRITE_JPEG_QUALITY), self.image_quality]
            bgr_rows = self.makerbot.yuv_to_bgr_rows(StringIO(image_data), width, height)
            arr = self.np.array(bgr_rows).reshape(height, width, 3)
            try:
                result, encoded_frame = self.cv2.imencode(self.image_extension, arr, encode_param)
            except Exception as e:
                self.logger.warning('Failed to encode camera frame: ' + str(e))
                result, encoded_frame = None, None
            if result:
                data = self.np.array(encoded_frame)
                string_data = data.tostring()
                # self.logger.debug("Successfully captured and encoded from" + str(capture))
                return string_data
        elif type == 2:
            return image_data
        self.logger.warning('Failed to get camera frame (wrong type): %i, %i, %i' % (width, height, type))
        return None
